<?php
/*
Plugin Name: Progress Bars
Version: 1.0
Plugin URI: http://needle-exchange.ca/categories/wp-progress-bars/
Description: Manages and displays your Works in Progress as handy little progress bars on your blog.
Author: Eve F
Author URI: http://needle-exchange.ca/

This plugin borrows from the Ubernyms plugin by Joel Bennett. Many thanks to him.
http://www.joelbennett.net/category/development/wordpress/ubernyms-plugin/

Copyright (c) 2006
Released under the GPL license
http://www.gnu.org/licenses/gpl.txt

    This file is part of WordPress.
    WordPress is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/*****************************************************************************
 *
 * crafty_load_settings
 * settings array consists of: sorting column (date started, type, %, title)
 *                             colour 1 (hex)
 *                             colour 2 (hex)
 *                             colour 3 (hex)
 *                             colour 4 (hex)
 *                             font_colour
 *                             picture (which picture is used for the bar)
 *                             fo_content
 *                             fo_cat
 *
 *****************************************************************************/

function crafty_load_settings() {
   global $crafty_settings;
   
   if( empty($crafty_settings) ) {
      $crafty_settings = get_option("crafty_settings");
      if( empty($crafty_settings) ) {
         $fo_post = '<a href="" title=""><img src="" alt="" /></a>

<strong>%TITLE%</strong>, started %START_DATE%, finished %FINISHED_DATE%
<strong>Pattern:</strong> <a title="" href=""></a>
<strong>Yarn:</strong>
<strong>Needles:</strong>
<strong>Dimensions:</strong>
<strong>Pattern Notes:</strong>
<strong>Would I knit it again?</strong>';
         $crafty_settings = array( "type", "653F90", "339999", "FCB89E", "FFDDEC", "330033", 0, $fo_post, NULL );
         update_option("crafty_settings", $crafty_settings);
      }
   }
}

function crafty_load () {
   global $crafty_projects;
   crafty_load_settings();

   if( empty($crafty_projects) ) {
      $crafty_projects = get_option("crafty_projects");
      // First, we define all the things we're going to replace, without using parenthesis or pipes (|)
      // ESPECIALLY note that they all end with commas EXCEPT the last one
      if( empty($crafty_projects) ) {
         // title, category_id, percentage, type, date started, date finished, FO post #, expected time span, due date
         $crafty_projects = array(
            new crafty_project("Jaywalkers (this is an example)", NULL, NULL, '30', "socks", time(), false, NULL, '30', time(), NULL, NULL )
         );

         update_option("crafty_projects", $crafty_projects );
      }
   }
}

function crafty_make_config_page() {
	if ( function_exists('add_submenu_page') )
		add_submenu_page('edit.php', 'Progress Bars', 'Progress Bars', 1, basename(__FILE__), 'crafty_config_page');
}


function crafty_config_page() {
   global $wpdb, $crafty_projects, $crafty_settings;
   crafty_load();
   
   if (isset($_GET['calendar'])) { ?>
      <div class="wrap">
      	<h2>Progress Bar Calendar (<a title="Preferences" href="<?php print $_SERVER['PHP_SELF'] . '?page=' . basename(__FILE__); ?>">View Preferences</a>)</h2>
      	<p>This is a basic calendar to help you plan out your projects and make sure you're not working on 15,000 things at the same time. A red block indicates a due date, and yellow blocks stretch back from the due date for as many days as you've set in your time span. If you've got lots of yellow blocks piling up in one spot, you've got a problem!</p>
      	<? $start_date = progress_get_first_day($_GET['calendar']);
      	progress_bars_display_calendar($start_date); ?>
      </div>
      <? return;
   }
   
   if (isset($_POST['crafty_update'])) {
      ?><div class="updated"><p><strong>Settings updated!</strong></p><?php
      $crafty_settings[0] = $_POST['sort'];		// sorting column (date started, type, %, title)
      $crafty_settings[1] = $_POST['colour1'];	// colour 1 (hex)
      $crafty_settings[2] = $_POST['colour2'];	// colour 2 (hex)
      $crafty_settings[3] = $_POST['colour3'];	// colour 3 (hex)
      $crafty_settings[4] = $_POST['colour4'];	// colour 4 (hex)
      $crafty_settings[5] = $_POST['font_colour'];	// font_colour (hex)
      $crafty_settings[6] = $_POST['picture'];	// picture (which picture is used for the bar)
      $crafty_settings[7] = $_POST['fo_content'];	// content for finished object post
      $crafty_settings[8] = $_POST['fo_cat'];	// category for finished object posts
      update_option("crafty_settings", $crafty_settings );

      $project_count = count($_POST['project_percent']);
      
      $project_finished_date = array();
      $project_due_date = array();
      $project_started_date = array();
      
      $project_stored = 0;
      $crafty_projects = array();
      
      for( $proj = 0; $proj < $project_count; $proj++ ) {
         // set all items with a title and without 'delete' checked
         if( $_POST['project_title'][$proj] > "" && !$_POST['project_delete'][$proj] ) {
            // deal with dates
            $project_finished_date[$proj] = mktime(12,0,0,
                       $_POST['project_finished_mo'][$proj],
                       $_POST['project_finished_day'][$proj],
                       $_POST['project_finished_yr'][$proj]);
            $project_due_date[$proj] = mktime(12,0,0,
                       $_POST['project_due_date_mo'][$proj],
                       $_POST['project_due_date_day'][$proj],
                       $_POST['project_due_date_yr'][$proj]);
            $project_started_date[$proj] = mktime(12,0,0,
                       $_POST['project_started_mo'][$proj],
                       $_POST['project_started_day'][$proj],
                       $_POST['project_started_yr'][$proj]);
            
            // set timespan to one if not entered
		    if ($_POST['project_time_span'][$proj] == '') {
		       $project_time_span = ceil(($project_due_date[$proj] -
		                                  $project_started_date[$proj])
		                                  / 86400);
		    } else {
		       $project_time_span = $_POST['project_time_span'][$proj];
		    }
		    
		    // set percent to zero if not entered
		    if (empty($_POST['project_percent'][$proj])) {
		       $project_percent = 0;
		    } else {
		       $project_percent = $_POST['project_percent'][$proj];
		    }
		    
		    // set finishing dates, post ids, whether things are finished
            if ($_POST['project_recently_finished'][$proj]) {
	           $project_finished_date[$proj] = time();
	           $fo_post_id = $wpdb->get_var("SELECT max(ID) FROM $wpdb->posts") + 1;
	           $project_percent = 100;
	        } else {
		       if (!$_POST['project_finished'][$proj]) {
		          $project_finished_date[$proj] = false;
	           }
		       $fo_post_id = $_POST['project_post_id'][$proj];
		    }
		    
		    // add array to list of projects
            $project_stored++;
            $crafty_projects[$proj] = new crafty_project(
               htmlentities(stripslashes($_POST['project_title'][$proj]), ENT_QUOTES),
               $_POST['project_hide'][$proj],
               htmlentities(stripslashes($_POST['project_cat'][$proj]), ENT_QUOTES),
               $project_percent,
               htmlentities(stripslashes($_POST['project_type'][$proj]), ENT_QUOTES),
               $project_started_date[$proj],
               $project_finished_date[$proj],
               $fo_post_id,
               $project_time_span,
               $project_due_date[$proj],
               $_POST['project_url'][$proj],
               $_POST['project_img_url'][$proj]);
            
            // create new FO post
            if (isset($_POST['project_recently_finished'][$proj])) {
	           create_finished_object_post($proj, $crafty_projects[$proj]);
	        }
         }
      }
      
      // make arrays to sort things
      // titles set to lowercase to sort case indiscriminately
      $title_sort = array();
      $finished_sort = array();
      foreach($crafty_projects as $current) {
         array_push($title_sort, strtolower($current->title));
         if (empty($current->finished)) {
            array_push($finished_sort, 0);
         } else {
            array_push($finished_sort, 1);
         }
      }
      
      $temp = $crafty_projects;
      array_multisort( $finished_sort,
                       SORT_ASC,
                       $title_sort,
                       SORT_ASC,
                       $temp );
      $crafty_projects = $temp;
      
      update_option("crafty_projects", $crafty_projects );
      print( "<p><b>$project_stored</b> projects stored." );
      ?></div><?php
   } ?>

   <div class="wrap">
      <h2>Progress Bar Preferences (<a title="Calendar" href="<?php print $_SERVER['PHP_SELF'] . '?page=' . basename(__FILE__) . '&calendar=current'; ?>">View Calendar</a>) (<a title="Display Options" href="#display">Display Options</a>)</h2>
      <p>You can use this system to keep track of all your projects. A <span class="overdue">pink highlight</span> means that your project is overdue.</p>
      <ul>
      	<li>Check "Complete" to denote that you've finished off a project. Every time you finish a project, a new post is created for you about the project.</li>
      	<li>Check "Remove" to delete any project; if you delete a project here, don't worry! The post you've written will be safe and sound.</li>
      	<li>Check "Hide" if you don't want to expose your project to prying public eyes just yet.</li>
      	<li>The "Time Alloted" section allows you to set a time span for your work that's independent of the start and due dates. You can set a specific number of days, or leave the field blank and the time span between the start and due dates will be calculated for you. To recalculate the time span (when changing the due date, for example), simply erase the numbers and they will be recalculated again.</li>
      	<li>The links in your progress bar list will default to the category you provide for each project. If you provide a link for a project, that link will override the link to the category.</li>
      </ul>

      <form method="post" action="<?php print $_SERVER['PHP_SELF'] . '?page=' . basename(__FILE__); ?>">
         <fieldset id="Current" class="progress"><legend>Current Projects</legend>
         <p>Your current and finished projects are listed below.</p>
         <table id="the-list-x" width="100%" cellpadding="3" cellspacing="2">
         <?php
            $craftycount = 0;
            foreach($crafty_projects as $current_project) {
               if( ($craftycount % 6) == 0 ) { ?>
            <tr>
         		<td colspan="6">
                  	<br />
         			<input type="submit" name="crafty_update" value="Set Options / Update Projects" />
         			<br /><br />
         		</td>
            </tr>
            <? display_crafty_table_head(); }
               $craftycount++; ?>
            <tr class="project-title<?php if ($current_project->finished) {
                        echo ' finished'; }
                      elseif (time() > $current_project->due_date) {
                        echo ' overdue'; } ?>">
               <td>
                  <?php if (!$current_project->finished) { /* unfinished */ ?>
                      <input name="project_title[]" size="15" type="text" value="<?php print($current_project->title);?>" />
                      ( <input type="checkbox" name="project_hide[<? echo $craftycount - 1; ?>]" value="true"<? if ($current_project->hide) { echo " checked"; } ?> /> Hide)
                  <? } else { /* finished */ ?>
                      <input name="project_title[]" type="hidden" value="<?php print($current_project->title);?>" />
                      <input name="project_post_id[<? echo $craftycount - 1; ?>]" type="hidden" value="<?php print($current_project->fo_post_id);?>" />
	                  <a title="<? echo $current_project->title; ?>" href="<? echo get_permalink($current_project->fo_post_id); ?>"><? echo $current_project->title; ?></a>
	                  (<a title="Edit Post" href="<? echo get_option('siteurl').'/wp-admin/post.php?action=edit&post='.$current_project->fo_post_id; ?>">Edit</a>)
                  <? } ?>
               </td>
               <td>
                  <input name="project_percent[]" size="3" type="text" value="<?php print($current_project->percent);?>" />
               </td>
               <td><select name="project_cat[]" width="150" style="width: 150px">
                     <?php display_categories_in_dropdown($current_project->cat); ?>
                  </select>
               </td>
               <td>
                  <input name="project_type[]" size="8" type="text" value="<?php print($current_project->type);?>" />
               </td>
               <td>
                  It'll take <input name="project_time_span[]" size="2" type="text" value="<?php print($current_project->time_span); ?>" /> day(s)
               </td>
               <td>
                  <input type="checkbox" name="project_delete[<? echo $craftycount - 1; ?>]" value="true" />
               </td>
            </tr>
            <tr>
               <td colspan="5">
                  Started:&nbsp;<?php date_form_display("project_started", $current_project->started, $craftycount - 1); ?>&nbsp;&nbsp;&nbsp;
                  Due Date:&nbsp;<?php date_form_display("project_due_date", $current_project->due_date, $craftycount - 1); ?>&nbsp;&nbsp;&nbsp;
                  Completed:&nbsp;<?php
               		if ($current_project->finished) { /* finished */
               		   date_form_display("project_finished", $current_project->finished, $craftycount - 1); ?>
                       <input type="hidden" name="project_finished[<? echo $craftycount - 1; ?>]" value="true" />
	              <?php } else { /* unfinished */ ?>
                    <input type="checkbox" name="project_recently_finished[<? echo $craftycount - 1; ?>]" value="true" />
                  <?php } ?><br />
                  Link:&nbsp;<input name="project_url[]" size="35" type="text" value="<?php print($current_project->url);?>" />&nbsp;&nbsp;&nbsp;
                  Image:&nbsp;<input name="project_img_url[]" size="35" type="text" value="<?php print($current_project->img_url);?>" />
               </td>
               <?php if($current_project->img_url) { ?><td align="right"><img src="<?php print($current_project->img_url);?>" width="40" alt="<?php print($current_project->title);?>" /></td><?php } ?>
            </tr>
            <? } ?>
            <tr>
         		<td colspan="6">
                  	<br />
         			<input type="submit" name="crafty_update" value="Set Options / Update Projects" />
         			<br /><br />
         		</td>
            </tr>
            <tr>
               <th colspan="6"><h3>Add new projects:</h3></th>
            </tr><?php display_crafty_table_head(); ?>
            <?php
            for($i=0; $i < 5; $i++) { $craftycount++;
            ?>               
            <tr class="alternate">
               <td>
                  <input name="project_title[]" size="15" type="text" value="" class="mandatory" />
                  ( <input type="checkbox" name="project_hide[<? echo $craftycount - 1; ?>]" value="true" /> Hide)
               </td>
               <td>
                  <input name="project_percent[]" size="3" type="text" value="" class="mandatory" />
               </td>
               <td><select name="project_cat[]" width="150" style="width: 150px">
                     <?php display_categories_in_dropdown($crafty_settings[8]); ?>
                  </select>
               </td>
               <td>
                  <input name="project_type[]" size="8" type="text" value="" />
               </td>
               <td>
	              It'll take <input name="project_time_span[]" size="2" type="text" value="" /> days
               </td>
               <td>&nbsp;</td>
            </tr>
            <tr>
               <td colspan="6">
                  Started:&nbsp;<?php date_form_display("project_started", time(), $craftycount - 1); ?>&nbsp;&nbsp;&nbsp;
                  Due Date:&nbsp;<?php date_form_display("project_due_date", time(), $craftycount - 1); ?>&nbsp;&nbsp;&nbsp;
                  Completed:&nbsp;<input type="checkbox" name="project_recently_finished[<? echo $craftycount - 1; ?>]" value="true" /><br />
               </td>
            </tr>
            <tr>
               <td colspan="6">
                  Link:&nbsp;<input name="project_url[]" size="35" type="text" value="" />&nbsp;&nbsp;&nbsp;
                  Image:&nbsp;<input name="project_img_url[]" size="35" type="text" value="" />
               </td>
            </tr>
            <?php } ?>
            
         </table>
         </fieldset>
      
      <br />
      <input type="submit" name="crafty_update" value="Set Options / Update Projects" />
      <br /><br />

      <h3 class="centered" id="display">Progress Bar Display</h3>
      <p>To display your progress bars, either add the progress bars widget or use the following code in your sidebar:
        <pre><code>&lt;?php if (function_exists('display_progress_bars')) { display_progress_bars(); } ?&gt;</code></pre>
        You can specify how you display your progress bars by passing parameters like so:<br />
        <pre>display_progress_bars('&lt;ul&gt;', '&lt;/ul&gt;', '&lt;li&gt;', '&lt;/li&gt;', '&lt;br /&gt;')</pre></p>
        <ul>
        	<li><strong>beginning</strong>: printed at the beginning of the list. Defaults to '&lt;ul&gt;'.</li>
        	<li><strong>end</strong>: printed at the end of the list. Defaults to '&lt;/ul&gt;'.</li>
        	<li><strong>before</strong>: printed before each list item. Defaults to '&lt;li&gt;'.</li>
        	<li><strong>after</strong>: printed after each list item. Defaults to '&lt;/li&gt;'.</li>
        	<li><strong>between</strong>: printed between the item title and progress bar. Defaults to '&lt;br /&gt;'.</li>
        </ul>
      <p>Because Wordpress does not allow PHP in a post, your progress bars will not display if you paste them into the body. However, to get around this you can install a plugin such as <a title="PHP Exec Wordpress Plugin" href="http://priyadi.net/archives/2005/03/02/wordpress-php-exec-plugin/">PHP Exec</a>, which will allow you to add PHP to the bodies of posts.</p>

         <fieldset id="Sort" class="progress"><legend>Sorting</legend>
         <p>Select the sorting order of the projects on your sidebar: by date started, by title, by percent completed, or by type (sweater, sock, shirt, etc).</p>
         <p><select name="sort">
         	<option value="started" <?php if($crafty_settings[0] == "started") {print 'selected="selected"';} ?>>Date Started</option>
         	<option value="title" <?php if($crafty_settings[0] == "title") {print 'selected="selected"';} ?>>Title</option>
         	<option value="percent" <?php if($crafty_settings[0] == "percent") {print 'selected="selected"';} ?>>Percent Completed</option>
         	<option value="type" <?php if($crafty_settings[0] == "type") {print 'selected="selected"';} ?>>Type</option>
         </select></p>
         </fieldset>
         <br />
         <input type="submit" name="crafty_update" value="Set Options / Update Projects" />
         <br /><br />
         
         <fieldset id="Colours" class="progress"><legend>Colours</legend>
         <p>As your progress continues to the end of your project, the colour of the bar will change to indicate how far along you are. Choose which colours you want to use for each stage (click "Set Options" to see changes):</p>
         <table cellpadding="4">
         	<tr>
         		<td>Colour between 0-24%:</td>
         		<td># <input type="text" name="colour1"<?php print(' value="'.$crafty_settings[1].'"'); ?> /></td>
         		<td><img src="<?php echo get_crafty_image_location(7,$crafty_settings[1], $crafty_settings[2], $crafty_settings[3], $crafty_settings[4], $crafty_settings[5], $crafty_settings[6]); ?>" alt="Progress Bar" /></td>
         	</tr>
         	<tr>
         		<td>Colour between 25%-49%:</td>
         		<td># <input type="text" name="colour2"<?php print(' value="'.$crafty_settings[2].'"'); ?> /></td>
         		<td><img src="<?php echo get_crafty_image_location(33,$crafty_settings[1], $crafty_settings[2], $crafty_settings[3], $crafty_settings[4], $crafty_settings[5], $crafty_settings[6]); ?>" alt="Progress Bar" /></td>
         	</tr>
         	<tr>
         		<td>Colour between 50-74%:</td>
         		<td># <input type="text" name="colour3"<?php print(' value="'.$crafty_settings[3].'"'); ?> /></td>
         		<td><img src="<?php echo get_crafty_image_location(65,$crafty_settings[1], $crafty_settings[2], $crafty_settings[3], $crafty_settings[4], $crafty_settings[5], $crafty_settings[6]); ?>" alt="Progress Bar" /></td>
         	</tr>
         	<tr>
         		<td>Colour between 75-100%:</td>
         		<td># <input type="text" name="colour4"<?php print(' value="'.$crafty_settings[4].'"'); ?> /></td>
         		<td><img src="<?php echo get_crafty_image_location(89,$crafty_settings[1], $crafty_settings[2], $crafty_settings[3], $crafty_settings[4], $crafty_settings[5], $crafty_settings[6]); ?>" alt="Progress Bar" /></td>
         	</tr>
         	<tr>
         		<td>Font Colour:</td>
         		<td># <input type="text" name="font_colour"<?php print(' value="'.$crafty_settings[5].'"'); ?> /></td>
         		<td><font color="#<?php print($crafty_settings[5]); ?>">Example: 100%</font></td>
         	<tr>
         </table>
         <p>Sample colours:</p>
         <table>
         	<tr>
         		<td bgcolor="#000000">&nbsp;&nbsp;</td><td>000000</td>
         		<td bgcolor="#000099">&nbsp;&nbsp;</td><td>000099</td>
         		<td bgcolor="#0000FF">&nbsp;&nbsp;</td><td>0000FF</td>
         		<td bgcolor="#009900">&nbsp;&nbsp;</td><td>009900</td>
         		<td bgcolor="#009999">&nbsp;&nbsp;</td><td>009999</td>
         		<td bgcolor="#0099FF">&nbsp;&nbsp;</td><td>0099FF</td>
         		<td bgcolor="#00FF00">&nbsp;&nbsp;</td><td>00FF00</td>
         		<td bgcolor="#00FF99">&nbsp;&nbsp;</td><td>00FF99</td>
         		<td bgcolor="#00FFFF">&nbsp;&nbsp;</td><td>00FFFF</td>
         	</tr>
         	<tr>
         		<td bgcolor="#990000">&nbsp;&nbsp;</td><td>990000</td>
         		<td bgcolor="#990099">&nbsp;&nbsp;</td><td>990099</td>
         		<td bgcolor="#9900FF">&nbsp;&nbsp;</td><td>9900FF</td>
         		<td bgcolor="#999900">&nbsp;&nbsp;</td><td>999900</td>
         		<td bgcolor="#999999">&nbsp;&nbsp;</td><td>999999</td>
         		<td bgcolor="#9999FF">&nbsp;&nbsp;</td><td>9999FF</td>
         		<td bgcolor="#99FF00">&nbsp;&nbsp;</td><td>99FF00</td>
         		<td bgcolor="#99FF99">&nbsp;&nbsp;</td><td>99FF99</td>
         		<td bgcolor="#99FFFF">&nbsp;&nbsp;</td><td>99FFFF</td>
         	</tr>
         	<tr>
         		<td bgcolor="#FF0000">&nbsp;&nbsp;</td><td>FF0000</td>
         		<td bgcolor="#FF0099">&nbsp;&nbsp;</td><td>FF0099</td>
         		<td bgcolor="#FF00FF">&nbsp;&nbsp;</td><td>FF00FF</td>
         		<td bgcolor="#FF9900">&nbsp;&nbsp;</td><td>FF9900</td>
         		<td bgcolor="#FF9999">&nbsp;&nbsp;</td><td>FF9999</td>
         		<td bgcolor="#FF99FF">&nbsp;&nbsp;</td><td>FF99FF</td>
         		<td bgcolor="#FFFF00">&nbsp;&nbsp;</td><td>FFFF00</td>
         		<td bgcolor="#FFFF99">&nbsp;&nbsp;</td><td>FFFF99</td>
         		<td bgcolor="#FFFFFF">&nbsp;&nbsp;</td><td>FFFFFF</td>
         	</tr>
         </table>
         </fieldset>
         <br />
         <input type="submit" name="crafty_update" value="Set Options / Update Projects" />
         <br /><br />
         
        <fieldset id="Picture" class="progress"><legend>Picture</legend>
        <?php $img_dir = realpath("..").'/wp-content/plugins/progress_bars/images'; ?>
		
		<table><tr><td width="180">
			<table>
		<?php if ($handle = opendir($img_dir)) {
			while (false !== ($file = readdir($handle))) {
				if ($file != '.' && $file != '..' && $file != 'template.jpg' && $file != 'template.png') {
					echo '<tr><label><td><input type="radio" name="picture" value="'.$file.'"';
					if($crafty_settings[6] == $file) echo ' checked="checked"';
					echo ' /></td><td><img src="'.get_crafty_image_location(100,$crafty_settings[1], $crafty_settings[2], $crafty_settings[3], $crafty_settings[4], $crafty_settings[5], $file).'" alt="Progress Bar" /></td></label></tr>';
         		}
         	}
         	closedir($handle);
         } ?>
         	</table>
         </td>
         <td><p>Choose the picture you want as your progress bar. To add your own picture, follow these directions:</p>
         <ol>
         	<li>Create an image in either PNG format or JPEG. (The progress bar will fill a space 100x14 pixels directly in the center of your image.)</li>
         	<li>Move your new image to <em>/wp-content/plugins/progress_bars/images/</em>.</li>
         	<li>Reload this page to select your new image.</li>
         </ol>
         <p>For transparent images, use PNG format (just use Photoshop or something like that to specify the alpha transparency). There are plenty of tutorials on the web if you don't know how to do this!</p>
         </td></tr></table>
         
         </fieldset>
         <br />
         <input type="submit" name="crafty_update" value="Set Options / Update Projects" />
         <br /><br />
         
         <fieldset id="FO_post" class="progress"><legend>Finished Object Posts</legend>
         <p>If you have a category for all your Finished Object posts, specify it here:</p>
         <p><select name="fo_cat">
         <?php display_categories_in_dropdown($crafty_settings[8]); ?>
         </select></p>
         <p>If you always write pretty much the same thing in each Finished Object post, write out the template here and it will be inserted into your post when you list your object as completed. You can edit it afterwards or delete the whole thing, if you like.</p>
         <p><textarea name="fo_content" cols="100" rows="5"><?php $msg = htmlentities(stripslashes($crafty_settings[7])); echo $msg; ?></textarea></p>
         <p>Insert the following item-specific Template Tags and they will be replaced with the proper information:<br />
            <strong>%TITLE%</strong><br />
            <strong>%START_DATE%</strong><br />
            <strong>%FINISHED_DATE%</strong><br />
            <strong>%CATEGORY%</strong><br />
            <strong>%DUE_DATE%</strong><br />
            <strong>%TYPE%</strong><br />
            <strong>%URL%</strong><br />
            <strong>%IMAGE%</strong></p>
         </fieldset>
         
         <br />
         <input type="submit" name="crafty_update" value="Set Options / Update Projects" />
         <br /><br />
      </form>
   </div>
   <?php
}

/* Simple class for projects, no handling for exceptional cases, don't "mess" with it :-P */
class crafty_project {
   var $title;
   var $hide;
   var $percent;
   var $cat;
   var $type;
   var $started;
   var $finished;
   var $fo_post_id;
   var $time_span;
   var $due_date;
   var $url;
   var $img_url;

   function crafty_project ( $title, $hide, $cat, $percent, $type, $started, $finished, $fo_post_id, $time_span, $due_date, $url, $img_url ) {
      $this->title = $title;
      $this->hide = $hide;
      $this->percent = $percent;
      $this->cat = $cat;
      $this->type = $type;
      $this->started = $started;
      $this->finished = $finished;
      $this->fo_post_id = $fo_post_id;
      $this->time_span = $time_span;
      $this->due_date = $due_date;
      $this->url = $url;
      $this->img_url = $img_url;
   }
}

function date_form_display ($input_name, $selected_date = '0', $array_no) {
	$day_sel = date("j", $selected_date);
	$mo_sel = date("n", $selected_date);
	$yr_sel = date("Y", $selected_date);
	
	echo '<select name="'.$input_name.'_day['.$array_no.']">';
	for ($day = 1; $day < 32; $day++) {
       echo '<option value="'.$day.'"';
       if($day_sel == $day) { echo ' selected="selected"'; }
       echo '>'.$day.'</option>';
    }
    echo '</select>&nbsp;';
    
	echo '<select name="'.$input_name.'_mo['.$array_no.']">';
	$months = array( 1 => "Jan",
					 2 => "Feb",
					 3 => "Mar",
					 4 => "Apr",
					 5 => "May",
					 6 => "Jun",
					 7 => "Jul",
					 8 => "Aug",
					 9 => "Sep",
					 10 => "Oct",
					 11 => "Nov",
					 12 => "Dec" );
	for ($mo = 1; $mo < 13; $mo++) {
       echo '<option value="'.$mo.'"';
       if($mo_sel == $mo) { echo ' selected="selected"'; }
       echo '>'.$months[$mo].'</option>';
    }
    echo '</select>&nbsp;';
    
	echo '<select name="'.$input_name.'_yr['.$array_no.']">';
	$this_year = date("Y");
	if ($yr_sel < ($this_year - 5)) {
		$first_year = $yr_sel - 5;
	} else {
		$first_year = $this_year - 5;
	}
	for ($yr = $first_year; $yr < $this_year + 11; $yr++) {
       echo '<option value="'.$yr.'"';
       if($yr_sel == $yr) { echo ' selected="selected"'; }
       echo '>'.$yr.'</option>';
    }
    echo '</select> ';
}

function display_crafty_table_head() {
?>
            <tr>
               <th>Title</th>
               <th>%</th>
               <th>Category</th>
               <th>Type</th>
               <th>Time Allotted</th>
               <th>Remove</th>
            </tr>
<?
}

function display_categories_in_dropdown($selected) {
	global $wpdb;
    $grab_cats = $wpdb->get_results("SELECT cat_ID, cat_name FROM $wpdb->categories WHERE category_parent = 0 ORDER BY cat_name ASC");
    foreach ($grab_cats as $cat) {
    	display_categories_in_dropdown_helper($cat, $selected, '-');
    }
}

function display_categories_in_dropdown_helper($cat, $selected, $before) {
	global $wpdb;
    echo '<option value="'.$cat->cat_ID.'"';
	if($cat->cat_ID == $selected) { echo ' selected="selected"'; }
	echo '>'.$before.' '.$cat->cat_name.'</option>';
	$grab_cats = $wpdb->get_results("SELECT cat_ID, cat_name FROM $wpdb->categories WHERE category_parent = $cat->cat_ID ORDER BY cat_name ASC");
	if ($grab_cats) {
		$before .= '-';
		foreach ($grab_cats as $cat) {
			display_categories_in_dropdown_helper($cat, $selected, $before);
		}
	}
}

function display_progress_bars($beginning = '<ul>', $ending = '</ul>', $before = '<li>', $after = '</li>', $between = '<br />') {
   global $crafty_projects, $crafty_settings;
   crafty_load();

   $sort_order = $crafty_settings[0];
   
   $order_array = array();
   foreach ($crafty_projects as $current_project) {
      array_push($order_array, $current_project->$sort_order);
   }
   
   $order = array_map('strtolower', $order_array);
   $crafty_list = $crafty_projects;
   array_multisort( $order, SORT_ASC, $crafty_list, SORT_ASC );
   
   echo $beginning;
   foreach ($crafty_list as $current_project) {
      if (!$current_project->finished && !$current_project->hide) {
		  $image_location = get_crafty_image_location($current_project->percent,
													  $crafty_settings[1],
													  $crafty_settings[2],
													  $crafty_settings[3],
													  $crafty_settings[4],
													  $crafty_settings[5],
													  $crafty_settings[6]);
		  if ($current_project->url) {
		  	$project_link = $current_project->url;
		  } else {
		  	$project_link = get_category_link($current_project->cat);
		  }
		  echo $before.'<a title="'.$current_project->title.'" href="'.$project_link.'">'.$current_project->title.'</a>'.$between;
		  echo '<a title="'.$current_project->title.'" href="'.$project_link.'"><img src="'.$image_location.'" alt="'.$current_project->title.'" /></a>'.$after;
      }
   }
   echo $ending;
}

function get_crafty_plugin_path() {
	return get_option('siteurl').'/wp-content/plugins/progress_bars/';
}

function get_crafty_image_location($percent, $colour1, $colour2, $colour3, $colour4, $font_colour, $picture = '0.png') {
   $image_location = get_crafty_plugin_path().'image.php';
   $image_location .= '?pict='.$picture;
   $image_location .= '&amp;colour1='.$colour1;
   $image_location .= '&amp;colour2='.$colour2;
   $image_location .= '&amp;colour3='.$colour3;
   $image_location .= '&amp;colour4='.$colour4;
   $image_location .= '&amp;font_colour='.$font_colour;
   $image_location .= '&amp;percent='.$percent;
   return $image_location;
}

function progress_bar_head() {
   $url = get_crafty_plugin_path().'extra-style.css';
   echo '<link rel="stylesheet" type="text/css" href="' . $url . '" />';
}

function create_finished_object_post($project_id, $project) {
	global $wpdb, $current_user, $crafty_settings;
	crafty_load();
	
	// load information
	$user = $current_user->id;
	$fo_cat = $crafty_settings[8];
	$fo_post = $current_user->fo_post_id;
	$message = $wpdb->escape(stripslashes(progress_bar_interpret($crafty_settings[7], $project)));
	$message_title = $wpdb->escape('Finished '.$project->title);
	
	// make post
	$now = date('Y-m-d H:i:s');
	$now_gmt = gmdate('Y-m-d H:i:s');
	$wpdb->query("INSERT INTO $wpdb->posts (ID, post_author, post_date, post_date_gmt, post_content, post_excerpt, post_title, post_status, post_category, post_name, post_modified, post_modified_gmt, comment_count, to_ping, pinged, post_content_filtered) VALUES ('$fo_post', '$current_user->id', '$now', '$now_gmt', '$message', '', '$message_title', 'draft', '0', '', '$now', '$now_gmt', '0', '', '', '')");
	
	$new_post_id = mysql_insert_id();
	
	// connect to the finished category
	$wpdb->query( "INSERT INTO $wpdb->post2cat (`rel_id`, `post_id`, `category_id`) VALUES ('', '$new_post_id', '$fo_cat')" );
	// connect to the project category
	if ($fo_cat != $project->cat) {
		$wpdb->query( "INSERT INTO $wpdb->post2cat (`rel_id`, `post_id`, `category_id`) VALUES ('', '$new_post_id', '$project->cat')" );
	}
}

function progress_bar_interpret($message, $project) {
	$replaceable = array ( '/%TITLE%/',
	                       '/%START_DATE%/',
	                       '/%FINISHED_DATE%/',
	                       '/%CATEGORY%/',
	                       '/%DUE_DATE%/',
	                       '/%TYPE%/',
	                       '/%URL%/',
	                       '/%IMAGE%/');
	
	$replace_with = array( $project->title,
	                       date("F jS, Y", $project->started),
	                       date("F jS, Y", $project->finished),
	                       $project->cat,
	                       date("F jS, Y", $project->due_date),
	                       $project->type,
	                       $project->url,
	                       $project->img_url);
	
	return preg_replace($replaceable, $replace_with, $message);
}

function progress_get_days_in_month($month, $year) {
	$month_to_days = array ( 0,
							 31,
							 28,
							 31,
							 30,
							 31,
							 30,
							 31,
							 31,
							 30,
							 31,
							 30,
							 31 );
	// Leap year stuff;
	// I'm not going to bother with the centennial business because
	// who's going to be using Wordpress in a hundred years?
	if ($month == 2 && $year % 4 == 0) {
		return 29;
	} else {
		return $month_to_days[$month];
	}
}

function progress_get_first_day($date) {
	// if it's the current (default) month
	if ($date == "current") {
		$start_date = mktime(12,0,0,date("n"),1,date("Y"));
	} else { // date is in YYYYMM format
		$year = substr($date, 0, 4);
		$month = substr($date, 4, 2);
		$start_date = mktime(12,0,0,$month,1,$year);
	}
	return $start_date;
}

function first_and_last_months($start_date) {
	$previous = $start_date - 1209600;
	$next = $start_date + 3628800;
	echo '<p class="centered">&laquo; <a title="Previous Month" href="';
	echo $_SERVER['PHP_SELF'].'?page='.basename(__FILE__).'&calendar=';
	echo date('Ym', $previous).'">'.date('F Y', $previous).'</a> &laquo; ';
	echo '&raquo; <a title="Previous Month" href="';
	echo $_SERVER['PHP_SELF'].'?page='.basename(__FILE__).'&calendar=';
	echo date('Ym', $next).'">'.date('F Y', $next).'</a> &raquo;</p>';
}

function progress_bars_round($num) {
	if ($num < 0) $num = 0;
	if ($num > 34) $num = 35;
	return floor($num);
}

function progress_bars_display_calendar($start_date) {
	global $crafty_projects, $crafty_settings;
	crafty_load();
	
	first_and_last_months($start_date);
	
	echo '<h3 class="centered">'.date("F Y", $start_date).'</h3>';
	$first_weekday = date("w", $start_date);
	$num_days = progress_get_days_in_month(date("n", $start_date), date("Y", $start_date));
	
	/* Let's create the calendar array!
	   The array is composed of 35 cells, each corresponding to a day
	   in the month (including those from previous and coming months).
	   The first cell in each day is a date stamp, and the next cells
	   are enumerated values - whether the corresponding project should be
	   worked on at that time. */
	$calendar = array();
	// figure out the date of the first calendar day
	// (one day is 86400 seconds)
	$calendar[0] = array();
	$calendar[0][0] = $start_date - (86400 * $first_weekday);
	
	for ($day = 1; $day < 35; $day++) {
		$calendar[$day] = array();
		$calendar[$day][0] = $calendar[$day - 1][0] + 86400;
	}
	
	// grab all uncompleted projects
	$unfinished = array();
	foreach ($crafty_projects as $current) {
		if (!$current->finished && $current->due_date >= $calendar[0][0]) {
			$beginning = $current->due_date - (86400 * $current->time_span);
			if ($beginning <= $calendar[34][0]) {
				array_push($unfinished, $current);
			}
		}
	}
	$num_unfinished = count($unfinished);
	
	/* load the calendar array with values */
	for ($p = 0; $p < $num_unfinished; $p++) {
		$ending = $unfinished[$p]->due_date;
		$ending_cell = progress_bars_round(($ending - $calendar[0][0]) / 86400);
		$beginning = $ending - (86400 * $unfinished[$p]->time_span);
		$beginning_cell = progress_bars_round(($beginning - $calendar[0][0]) / 86400);
		
		for ($day = $beginning_cell; $day < $ending_cell; $day++) {
			$calendar[$day][$p+1] = 1;
		}
		// due date is a different colour
		if ($ending_cell < 35) { $calendar[$ending_cell][$p+1] = 2; }
	} ?>
	
	<table id="the-list-x" width="100%" cellpadding="3" cellspacing="2">
		<tr>
			<th></th>
			<th class="equal_col">Sunday</th>
			<th class="equal_col">Monday</th>
			<th class="equal_col">Tuesday</th>
			<th class="equal_col">Wednesday</th>
			<th class="equal_col">Thursday</th>
			<th class="equal_col">Friday</th>
			<th class="equal_col">Saturday</th>
		</tr>
		<? for ($week = 0; $week < 5; $week++) {
			echo '<tr class="alternate"><td><!-- blank --></td>';
			for ($weekday = 0; $weekday < 7; $weekday++) {
				$day = ($week * 7) + $weekday;
				echo '<td';
				if (date('dm') == date('dm', $calendar[$day][0])) {
					echo ' class="today"';
				}
				echo '>'.date('j', $calendar[$day][0]).'</td>';
			}
			echo '</tr>';
			
			for ($proj = 0; $proj < $num_unfinished; $proj++) {
				echo '<tr>';
				echo '<td>'.$unfinished[$proj]->title.'</td>';
				for ($weekday = 0; $weekday < 7; $weekday++) {
					$day = ($week * 7) + $weekday;
					echo '<td class="progress-cal-'.$calendar[$day][$proj+1].'">&nbsp;</td>';
				}
				echo '</tr>';
			}
		} ?>
		
	</table>
	
	<? first_and_last_months($start_date);
}

// Put functions into one big function we'll call at the plugins_loaded
// action. This ensures that all required plugin functions are defined.
function widget_progress_bars_init() {

	// Check for the required plugin functions. This will prevent fatal
	// errors occurring when you deactivate the dynamic-sidebar plugin.
	if ( !function_exists('register_sidebar_widget') )
		return;

	function widget_progress_bars($args) {
		
		// $args is an array of strings that help widgets to conform to
		// the active theme: before_widget, before_title, after_widget,
		// and after_title are the array keys. Default tags: li and h2.
		extract($args);

		// Each widget can store its own options. We keep strings here.
		$title = get_option('widget_progress_bars');
		
		// These lines generate our output. Widgets can be very complex
		// but as you can see here, they can also be very, very simple.
		echo $before_widget . $before_title . $title . $after_title;
		display_progress_bars();
		echo $after_widget;
	}

	// This is the function that outputs the form to let the users edit
	// the widget's title. It's an optional feature that users cry for.
	function widget_progress_bars_control() {

		// Get our options and see if we're handling a form submission.
		$title = get_option('widget_progress_bars');
		if ( !$title )
			$title = 'Progress Bars';
		if ( $_POST['progress_bars-submit'] ) {

			// Remember to sanitize and format use input appropriately.
			$title = strip_tags(stripslashes($_POST['progress_bars-title']));
			update_option('widget_progress_bars', $title);
		}

		// Be sure you format your options to be valid HTML attributes.
		$title = htmlspecialchars($title, ENT_QUOTES);
		
		// Here is our little form segment. Notice that we don't need a
		// complete form. This will be embedded into the existing form.
		echo '<p style="text-align:right;"><label for="progress_bars-title">' . __('Title:') . ' <input style="width: 200px;" id="progress_bars-title" name="progress_bars-title" type="text" value="'.$title.'" /></label></p>';
		echo '<input type="hidden" id="progress_bars-submit" name="progress_bars-submit" value="1" />';
	}
	
	// This registers our widget so it appears with the other available
	// widgets and can be dragged and dropped into any active sidebars.
	register_sidebar_widget(array('Progress Bars', 'widgets'), 'widget_progress_bars');

	// This registers our optional widget control form. Because of this
	// our widget will have a button that reveals a 300x100 pixel form.
	register_widget_control(array('Progress Bars', 'widgets'), 'widget_progress_bars_control', 300, 100);
}

// Run our code later in case this loads prior to any required plugins.
add_action('widgets_init', 'widget_progress_bars_init');

// add the config menu
add_action('admin_menu', 'crafty_make_config_page');
add_action('admin_head', 'progress_bar_head');

?>